# -*- coding: utf-8 -*-
import datetime
import logging
import tempfile
import zipfile
from zipfile import ZIP_DEFLATED
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.db import connection
from django.http import HttpResponse
from django.shortcuts import render
from django.views.decorators.http import require_POST
from dates2utils.dates2export import (
    xml_teilnehmerDatenImport,
    update_by_timeinterval
)
from reportserver.lib.io.requests import (
    check_date,
    convert_german_date_to_iso
)


logger = logging.getLogger(__name__)


def dict_cursor(cursor):
    description = [x[0] for x in cursor.description]
    for row in cursor:
        yield dict(zip(description, row))


def extract_program_name(beneficiary_id):
    """Extract the program name from beneficiary_id.

    By definition the first section, seperated by '.' (dot).
    """
    return beneficiary_id.split('.')[0]


def update_indicator_name(row, indicators):
    """It is unclear why the indicator name update is necessary.

    This should be explained.
    """
    r = {}
    for key in row:
        if key.upper() in indicators:
            r[key.upper()] = row[key]
        elif key.upper().replace("_", ".") in indicators:
            r[key.upper().replace("_", ".")] = row[key]
        else:
            r[key] = row[key]
    return r


def personal_id_to_instanceid(value, separator="-", merger="."):
    split_value = value.split(separator)
    return merger.join(split_value[0:2])


def stringify_pair(p1, p2):
    return "%s;%s" % (p1, p2)


def get_full_beneficiary(string1, beneficiaries):
    # calc the year number of the beneficiary_id
    for beneficiary in beneficiaries:
        instance_id_with_underscores = beneficiary[0]
        if instance_id_with_underscores.startswith(string1):
            return instance_id_with_underscores
    return string1


def get_xml_file_name(orig_bene_id, time_str, rc=None):
    bene_id_with_underscores = orig_bene_id.replace(".", "_")
    if rc:
        return u"{}_{}_{}.xml".format(bene_id_with_underscores, rc, time_str)
    else:
        return u"{}_{}.xml".format(bene_id_with_underscores, time_str)


def get_GUID(orig_bene_id, session, rc=None):
    if rc:
        return session.get("dates2_lut")[stringify_pair(orig_bene_id, rc)]
    else:
        return session.get("dates2_lut")[stringify_pair(orig_bene_id, "*")]


def filter_only_rc(rows, rc):
    return [row for row in rows if row.get('regioncategory_id') == rc]


def handle_beneficiary(beneficiary_id, cursor, start, end, time_str, zip1,
                       session):
    cursor.execute("SELECT * FROM eval_virtual_participations"
                   " WHERE originating_beneficiary_id=%s ;",
                   [beneficiary_id])
    rows = [x for x in dict_cursor(cursor)]
    rows = update_by_timeinterval(
        rows, start, end,
        {'RI': settings.DATES2_RI_INDICATORS})
    out = []
    oi_indicators = settings.DATES2_OI_INDICATORS
    ri_indicators = settings.DATES2_RI_INDICATORS
    indicators = oi_indicators + ri_indicators
    rows_updated = [update_indicator_name(row, indicators)
                    for row in rows]
    if settings.PROGRAM == "EC":
        for rc in [1, 2, 3, 4]:
            out = []
            rc_filtered_rows = filter_only_rc(rows_updated, rc)
            if not rc_filtered_rows:
                continue
            xml_teilnehmerDatenImport(out,
                                      get_GUID(beneficiary_id, session, rc),
                                      rc_filtered_rows,
                                      oi_indicators, ri_indicators)
            output = u"\n".join(out)
            zip1.writestr(get_xml_file_name(beneficiary_id, time_str, rc),
                          output)
    else:
        out = []
        xml_teilnehmerDatenImport(out, get_GUID(beneficiary_id, session),
                                  rows_updated,
                                  oi_indicators, ri_indicators)
        output = u"\n".join(out)
        zip1.writestr(get_xml_file_name(beneficiary_id, time_str), output)


def get_date_from(request, name):
    return datetime.datetime.strptime(check_date(convert_german_date_to_iso(
        request.POST.get(name))), '%Y-%m-%d').date()


def handle_insert_lists(cursor, start, end, time_str, session, main_zip):
    # handle dates2 insert xml part
    cursor.execute("SELECT DISTINCT originating_beneficiary_id "
                   "FROM eval_virtual_participations;")
    beneficiary_ids = [row[0] for row in cursor.fetchall()]
    with tempfile.TemporaryFile() as ziptmpfile2:
        with zipfile.ZipFile(ziptmpfile2, "w", ZIP_DEFLATED) as zip2:
            # for every b: 1. get rows 2. xml them
            for beneficiary_id in beneficiary_ids:
                handle_beneficiary(beneficiary_id, cursor, start, end,
                                   time_str, zip2, session)
        ziptmpfile2.seek(0)
        progname = get_progname_by_beneficiaries(beneficiary_ids)
        main_zip.writestr(create_normal_zip_arcname(progname, time_str),
                          ziptmpfile2.read())


def create_normal_zip_arcname(program, time_str):
    """create the arcname for the normal xml zip."""
    return "Teilnehmerlisten-%s_%s.zip" % (program, time_str)


# code for generation of the remove lists
def create_rm_list_xml_arcname(beneficiary, number, time_str):
    """create the arcname for a rm list xml."""
    return "%s-%d_%s.xml" % (beneficiary, number, time_str)


def create_rm_lists_zip_arcname(program, number, time_str, region_category):
    """create the arcname for a rm lists zip."""
    if region_category == "*":
        return "Loeschlisten-%s_%s-%d.zip" % (program, time_str, number)
    return "Loeschlisten-%s-%s_%s-%d.zip" % (program, region_category,
                                             time_str, number)


def chunk_2(seq, n):
    """Split sequence seq in max n large chunks."""
    i = 0
    while seq[i:]:
        yield seq[i:i + n]
        i += n


def generate_rm_list_xml(participant_uuid, personal_id, instance_guid):
    out = []
    xml_teilnehmerDatenImport(out, instance_guid, [], [], [],
                              loeschliste=[{
                                  "participant_uuid": participant_uuid,
                                  "personal_id": personal_id}])
    output = u"\n".join(out)
    return output


MAX_FILES_IN_ZIP = 65000


def handle_rm_lists(cursor, time_str, main_zip, session):
    """Handle the create of the rm lists (XML-Löschlisten), write the Zips
    and insert them into the main_zip."""
    cursor.execute("SELECT uuid, personal_id, originating_beneficiary_id"
                   " FROM participant_ids ;")
    rows = [row for row in cursor.fetchall()]
    if settings.PROGRAM == "EC":
        region_categories = ["1", "2", "3", "4"]
    else:
        region_categories = ["*"]
    for region_category in region_categories:
        chunks = chunk_2(rows, MAX_FILES_IN_ZIP)
        nr_entries = 1
        nr_chunks = 1
        for chunk in chunks:
            with tempfile.TemporaryFile() as ziptmpfile:
                ZDE = ZIP_DEFLATED
                with zipfile.ZipFile(ziptmpfile, "w", ZDE) as secondary_zip:
                    for participant_uuid, personal_id, orig_bene_id in chunk:
                        # Generate one remove list for one participant
                        if settings.PROGRAM == "EC":
                            instance_guid = get_GUID(orig_bene_id, session,
                                                     region_category)
                            instance_id_full = (orig_bene_id + "-" +
                                                region_category)
                        else:
                            instance_guid = get_GUID(orig_bene_id, session)
                            instance_id_full = orig_bene_id
                        xml = generate_rm_list_xml(participant_uuid,
                                                   personal_id, instance_guid)
                        arcname = create_rm_list_xml_arcname(
                            instance_id_full, nr_entries, time_str)
                        secondary_zip.writestr(arcname, xml)
                        nr_entries += 1
                ziptmpfile.seek(0)
                zip_str = ziptmpfile.read()
                progname = get_progname_by_beneficiaries(
                    [bene[0] for bene in session['beneficiary']])
                zip_arcname = create_rm_lists_zip_arcname(progname,
                                                          nr_chunks,
                                                          time_str,
                                                          region_category)
                main_zip.writestr(zip_arcname, zip_str)
            nr_chunks += 1


# end rm list stuff

def get_progname_by_beneficiaries(beneficiary_ids):
    if beneficiary_ids:
        return extract_program_name(beneficiary_ids[0])
    else:
        return settings.PROGRAM


@login_required
@require_POST
def export_dates2(request):
    time_str = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
    try:
        start = get_date_from(request, 'start_date')
        end = get_date_from(request, 'end_date')
    except ValueError:
        return overview(request)

    with connection.cursor() as cursor:
        with tempfile.TemporaryFile() as ziptmpfile:
            with zipfile.ZipFile(ziptmpfile, "w", ZIP_DEFLATED) as zip1:

                handle_rm_lists(cursor, time_str, zip1, request.session)
                handle_insert_lists(cursor, start, end, time_str,
                                    request.session, zip1)
            ziptmpfile.seek(0)
            zip_str = ziptmpfile.read()

    program_name = get_progname_by_beneficiaries(
        [bene[0] for bene in request.session.get('beneficiary')])
    response = HttpResponse(zip_str, content_type="application/zip")
    response['Content-Disposition'] = ('attachment; filename="%s_%s.zip"' %
                                       (program_name, time_str))
    logger.info("'%s' requested a dates2export.", request.user.username)
    return response


@login_required
def overview(request):
    default_end_date = settings.DATES2_DEFAULT_END_DATE
    return render(request, 'dates2export.html',
                  {"default_end_date": default_end_date})
